<?php

namespace Drupal\phonepay_payment;

use Drupal\phonepay_payment\Methods\CustomCheckout;
use Drupal\phonepay_payment\Methods\RecurringCheckout;
use Drupal\phonepay_payment\Methods\StandardCheckout;

/**
 * Class to provide PhonePe basic fucntion.
 */
class PhonePe {

  /**
   * {@inheritdoc}
   */
  private static $instance;

  /**
   * {@inheritdoc}
   */
  private $config;

  /**
   * {@inheritdoc}
   */
  private function __construct($config) {
    $this->config = $config;
  }

  /**
   * Get the instance of the PhonePe class.
   *
   * @return PhonePe
   *   Return PhonePe Instance.
   */
  public static function getInstance() {
    if (NULL === static::$instance) {
      throw new \Exception("PhonePe not initialized");
    }

    return static::$instance;
  }

  /**
   * Initialize the PhonePe class.
   *
   * @param string $merchantId
   *   MerchantId.
   * @param string $merchantUserId
   *   MerchantUserId.
   * @param string $salt_key
   *   Salt_key.
   * @param string $salt_index
   *   Salt_index.
   * @param string $redirectUrl
   *   RedirectUrl.
   * @param string $callbackUrl
   *   CallbackUrl.
   * @param string $mode
   *   Mode.
   * @param string $redirectMode
   *   RedirectMode.
   *
   * @return PhonePe
   *   Return PhonePe Instance.
   */
  public static function init($merchantId, $merchantUserId, $salt_key, $salt_index, $redirectUrl = "", $callbackUrl = "", $mode = "DEV", $redirectMode = "redirect") {
    if (NULL === static::$instance) {
      if ($mode == "PROD") {
        $host = "https://api.phonepe.com/apis/hermes";
      }
      else {
        $host = "https://api-preprod.phonepe.com/apis/pg-sandbox";
      }

      static::$instance = new static(
        [
          "MERCHANT_ID" => $merchantId,
          "MERCHANT_USER_ID" => $merchantUserId,
          "SALT_KEY" => $salt_key,
          "SALT_INDEX" => $salt_index,
          "REDIRECT_URL" => $redirectUrl,
          "CALLBACK_URL" => $callbackUrl,
          "HOST" => $host,
          "REDIRECT_MODE" => $redirectMode,
        ]
      );
    }

    return static::$instance;
  }

  /**
   * Get the config of the PhonePe class.
   *
   * @param string $key
   *   Key.
   *
   * @return array
   *   Array.
   */
  public function getConfig($key = NULL) {
    if ($key) {
      return $this->config[$key];
    }

    return $this->config;
  }

  /**
   * {@inheritdoc}
   */
  public function standardCheckout() {
    return new StandardCheckout();
  }

  /**
   * {@inheritdoc}
   */
  public function customCheckout() {
    return new CustomCheckout();
  }

  /**
   * {@inheritdoc}
   */
  public function recurringCheckout() {
    return new RecurringCheckout();
  }

}
